/*  spiasm.h

 Uses spin file xSPIAsm.spin, modified from SPI_Asm.spin propeller tool library object.
  Modifications:
  1.  Uses 'command' with PASM to load ClockDelay and ClockState variables into cog.
  2.  spi_start parameter 'clockdelay_ns' is in nanosecs and function calculates ClockDelay value
 
  device->command is either 0 or packed with 2 or 3 numbers to be used by pasm.
     command = 0 -- pasm in idle loop
     in spi_start, the number 4 is shifted left 16 bits to indicate 'startup'
                              the ClockDelay value is shifted left 2 bits 
                              the ClockState value is in bit 0.  All 3 values are or'd.
     in spi_in the number 2 is shifted left 16 bits, this is or'd with the address of the Dpin
     in spi_out, the number 1 is shifted left 16 bits, and or'd with the address of the Dpin 

*/


#include "simpletools.h"                      // Library includes

//      **************** SPI constants ****************
        // For receiving data from SPI device (spi_in)
#define MPRE 0        //  Most Significant Bit first - data valid before clock
#define LPRE 1         //  Least Significant Bit first - data valid before clock
#define MPOST 2      //  Most Significant Bit first - data valid after clock
#define LPOST 3      //  Least Significant Bit first - data valid after clock

        // For sending data to SPI device (spi_out)
#define LFIRST 4     //  Least Significant Bit first ; data valid after clock
#define MFIRST 5    //  Most Significant Bit first ; data valid after clock
 
typedef struct spi_struct               // Structure contains varaibles
 {                                             // for cogs to share + cog & stack
   volatile int command;           
   int Dpin;
   int Cpin;
   volatile int Mode;
   volatile int Bits;
   volatile int Value;
   volatile int Flag;
   int cog;                                    // Remembers cog for spi_stop 
 } spia_t;                                    // Data type for struct

// function prototypes
 
    // these prototypes go in library h file (spi.h)
spia_t *spi_start(int clockdelay_ns, int clockstate);
  
void spi_stop(spia_t *device); 

void spi_out(spia_t *device, int DQ, int CLK, int MD, int BIT, int VLU);
  
int spi_in(spia_t *device,int DQ, int CLK, int MD, int BIT);

